;+
;
; @file_comments compute the latitudes of the n160 gaussian grid. See:
; http://www.ecmwf.int/products/data/technical/gaussian/n160FIS.html
;
; @categories grid
;
; @examples 
; IDL>lat=n160gaussian()
;
; @returns a 1d array
;
; @history
;      Sebastien Masson (smasson\@lodyc.jussieu.fr)
;      June 2004
;-
;
FUNCTION n160gaussian
;
;
  compile_opt idl2, strictarrsubs
;
;          latitude   reduced     regular  latitude
;           number     points      points
;          -------    -------     -------  --------
  n160 = [    1,         18,        640,  89.57009, $
              2,         25,        640,  89.01318, $
              3,         36,        640,  88.45297, $
              4,         40,        640,  87.89203, $
              5,         45,        640,  87.33080, $
              6,         50,        640,  86.76944, $
              7,         60,        640,  86.20800, $
              8,         64,        640,  85.64651, $
              9,         72,        640,  85.08499, $
              10,         72,        640,  84.52345, $
              11,         80,        640,  83.96190, $
              12,         90,        640,  83.40033, $
              13,         90,        640,  82.83876, $
              14,         96,        640,  82.27718, $
              15,        108,        640,  81.71559, $
              16,        120,        640,  81.15400, $
              17,        120,        640,  80.59240, $
              18,        125,        640,  80.03080, $
              19,        128,        640,  79.46920, $
              20,        135,        640,  78.90760, $
              21,        144,        640,  78.34600, $
              22,        150,        640,  77.78439, $
              23,        160,        640,  77.22278, $
              24,        160,        640,  76.66117, $
              25,        180,        640,  76.09956, $
              26,        180,        640,  75.53795, $
              27,        180,        640,  74.97634, $
              28,        192,        640,  74.41473, $
              29,        192,        640,  73.85311, $
              30,        200,        640,  73.29150, $
              31,        216,        640,  72.72988, $
              32,        216,        640,  72.16827, $
              33,        225,        640,  71.60665, $
              34,        225,        640,  71.04504, $
              35,        240,        640,  70.48342, $
              36,        240,        640,  69.92181, $
              37,        243,        640,  69.36019, $
              38,        250,        640,  68.79857, $
              39,        256,        640,  68.23695, $
              40,        270,        640,  67.67534, $
              41,        270,        640,  67.11372, $
              42,        288,        640,  66.55210, $
              43,        288,        640,  65.99048, $
              44,        288,        640,  65.42886, $
              45,        300,        640,  64.86725, $
              46,        300,        640,  64.30563, $
              47,        320,        640,  63.74401, $
              48,        320,        640,  63.18239, $
              49,        320,        640,  62.62077, $
              50,        320,        640,  62.05915, $
              51,        324,        640,  61.49753, $
              52,        360,        640,  60.93591, $
              53,        360,        640,  60.37429, $
              54,        360,        640,  59.81267, $
              55,        360,        640,  59.25105, $
              56,        360,        640,  58.68943, $
              57,        360,        640,  58.12781, $
              58,        375,        640,  57.56619, $
              59,        375,        640,  57.00457, $
              60,        375,        640,  56.44295, $
              61,        384,        640,  55.88133, $
              62,        384,        640,  55.31971, $
              63,        400,        640,  54.75809, $
              64,        400,        640,  54.19647, $
              65,        400,        640,  53.63485, $
              66,        405,        640,  53.07323, $
              67,        432,        640,  52.51161, $
              68,        432,        640,  51.94999, $
              69,        432,        640,  51.38837, $
              70,        432,        640,  50.82675, $
              71,        432,        640,  50.26513, $
              72,        450,        640,  49.70351, $
              73,        450,        640,  49.14189, $
              74,        450,        640,  48.58026, $
              75,        450,        640,  48.01864, $
              76,        480,        640,  47.45702, $
              77,        480,        640,  46.89540, $
              78,        480,        640,  46.33378, $
              79,        480,        640,  45.77216, $
              80,        480,        640,  45.21054]
  n160 = [n160, $
          81,        480,        640,  44.64892, $
          82,        480,        640,  44.08730, $
          83,        500,        640,  43.52567, $
          84,        500,        640,  42.96405, $
          85,        500,        640,  42.40243, $
          86,        500,        640,  41.84081, $
          87,        500,        640,  41.27919, $
          88,        512,        640,  40.71757, $
          89,        512,        640,  40.15595, $
          90,        540,        640,  39.59433, $
          91,        540,        640,  39.03270, $
          92,        540,        640,  38.47108, $
          93,        540,        640,  37.90946, $
          94,        540,        640,  37.34784, $
          95,        540,        640,  36.78622, $
          96,        540,        640,  36.22460, $
          97,        540,        640,  35.66298, $
          98,        576,        640,  35.10136, $
          99,        576,        640,  34.53973, $
          100,        576,        640,  33.97811, $
          101,        576,        640,  33.41649, $
          102,        576,        640,  32.85487, $
          103,        576,        640,  32.29325, $
          104,        576,        640,  31.73163, $
          105,        576,        640,  31.17000, $
          106,        576,        640,  30.60838, $
          107,        576,        640,  30.04676, $
          108,        600,        640,  29.48514, $
          109,        600,        640,  28.92352, $
          110,        600,        640,  28.36190, $
          111,        600,        640,  27.80028, $
          112,        600,        640,  27.23865, $
          113,        600,        640,  26.67703, $
          114,        600,        640,  26.11541, $
          115,        600,        640,  25.55379, $
          116,        600,        640,  24.99217, $
          117,        640,        640,  24.43055, $
          118,        640,        640,  23.86892, $
          119,        640,        640,  23.30730, $
          120,        640,        640,  22.74568, $
          121,        640,        640,  22.18406, $
          122,        640,        640,  21.62244, $
          123,        640,        640,  21.06082, $
          124,        640,        640,  20.49919, $
          125,        640,        640,  19.93757, $
          126,        640,        640,  19.37595, $
          127,        640,        640,  18.81433, $
          128,        640,        640,  18.25271, $
          129,        640,        640,  17.69109, $
          130,        640,        640,  17.12946, $
          131,        640,        640,  16.56784, $
          132,        640,        640,  16.00622, $
          133,        640,        640,  15.44460, $
          134,        640,        640,  14.88298, $
          135,        640,        640,  14.32136, $
          136,        640,        640,  13.75973, $
          137,        640,        640,  13.19811, $
          138,        640,        640,  12.63649, $
          139,        640,        640,  12.07487, $
          140,        640,        640,  11.51325, $
          141,        640,        640,  10.95162, $
          142,        640,        640,  10.39000, $
          143,        640,        640,   9.82838, $
          144,        640,        640,   9.26676, $
          145,        640,        640,   8.70514, $
          146,        640,        640,   8.14352, $
          147,        640,        640,   7.58189, $
          148,        640,        640,   7.02027, $
          149,        640,        640,   6.45865, $
          150,        640,        640,   5.89703, $
          151,        640,        640,   5.33541, $
          152,        640,        640,   4.77379, $
          153,        640,        640,   4.21216, $
          154,        640,        640,   3.65054, $
          155,        640,        640,   3.08892, $
          156,        640,        640,   2.52730, $
          157,        640,        640,   1.96568, $
          158,        640,        640,   1.40405, $
          159,        640,        640,   0.84243, $
          160,        640,        640,   0.28081]
;
  n160 = reform(n160, 4, 160, /over)
  n160 = reform(n160[3, *], /over)
  n160 = [n160, -reverse(n160)]

  return, n160
end
