;+
;
; @file_comments compute the latitudes of the n48 gaussian grid. See:
; http://www.ecmwf.int/products/data/technical/gaussian/n48FIS.html
;
; @categories grid
;
; @examples
; IDL> lat=n48gaussian()
;
; @returns a 1d array
;
; @history
;      Sebastien Masson (smasson\@lodyc.jussieu.fr)
;      June 2004
;-
;
FUNCTION n48gaussian
;
;
  compile_opt idl2, strictarrsubs
;
;          latitude   reduced     regular  latitude
;           number     points      points
;          -------    -------     -------  --------
  n48 = [     1,         20,        192,  88.57216, $
              2,         25,        192,  86.72253, $
              3,         36,        192,  84.86197, $
              4,         40,        192,  82.99894, $
              5,         45,        192,  81.13497, $
              6,         50,        192,  79.27055, $
              7,         60,        192,  77.40588, $
              8,         60,        192,  75.54106, $
              9,         72,        192,  73.67613, $
              10,         75,        192,  71.81113, $
              11,         80,        192,  69.94608, $
              12,         90,        192,  68.08099, $
              13,         96,        192,  66.21587, $
              14,        100,        192,  64.35073, $
              15,        108,        192,  62.48557, $
              16,        120,        192,  60.62039, $
              17,        120,        192,  58.75520, $
              18,        120,        192,  56.89001, $
              19,        128,        192,  55.02480, $
              20,        135,        192,  53.15959, $
              21,        144,        192,  51.29437, $
              22,        144,        192,  49.42915, $
              23,        160,        192,  47.56392, $
              24,        160,        192,  45.69869, $
              25,        160,        192,  43.83345, $
              26,        160,        192,  41.96822, $
              27,        160,        192,  40.10297, $
              28,        180,        192,  38.23773, $
              29,        180,        192,  36.37249, $
              30,        180,        192,  34.50724, $
              31,        180,        192,  32.64199, $
              32,        180,        192,  30.77674, $
              33,        192,        192,  28.91149, $
              34,        192,        192,  27.04623, $
              35,        192,        192,  25.18098, $
              36,        192,        192,  23.31573, $
              37,        192,        192,  21.45047, $
              38,        192,        192,  19.58521, $
              39,        192,        192,  17.71996, $
              40,        192,        192,  15.85470, $
              41,        192,        192,  13.98944, $
              42,        192,        192,  12.12418, $
              43,        192,        192,  10.25892, $
              44,        192,        192,   8.39366, $
              45,        192,        192,   6.52840, $
              46,        192,        192,   4.66314, $
              47,        192,        192,   2.79788, $
              48,        192,        192,   0.93262]
;
  n48 = reform(n48, 4, 48, /over)
  n48 = reform(n48[3, *], /over)
  n48 = [n48, -reverse(n48)]

  return, n48
end
